"""
    parse_onda("name-of-calibration-file.txt")

Translates calibration text files from Onda
into the format required by this package.
Output is a .jl file written to the current active directory.
Running the new file will load the calibration data to be used by HydrophoneCalibrations.

"""
function parse_onda(filename)
    info = Dict()
    read_file_as_strings!(info, filename)
    parse_header!(info)
    get_data!(info)
    parse_data!(info)
    get_raw_header!(info)
    create_device_name!(info)
    savepath = write_to_file!(info)
    return savepath
end

function read_file_as_strings!(info, filename)
    data_in = readlines(filename)
    if length(data_in) == 1
        data_in = split(data_in[1], '\r')
    end
    info["data_in"] = data_in
    return nothing
end

function parse_header!(info)
    data_in = info["data_in"]
    nlines = 0
    for line in data_in
        nlines += 1
        store_field(info, "MFG", line)
        store_field(info, "MODEL", line)
        store_field(info, "SN", line)
        store_field(info, "AMP_GAIN_NOM_DB", line)
        store_field(info, "FILE_NAME", line)
        store_field(info, "AMP_CAL_DATE", line)
        store_field(info, "HYD_APERTURE_NOM_UM", line)
        store_field(info, "Calibration_DATE", line)
        store_field(info, "DATA_FIELDS", line)
        if is_header_end(line)
            break
        end
    end
    info["num_header_lines"] = nlines
    set_date!(info)
    format_manufacturing!(info)

    return nothing
end

function store_field(info, field, line)
    if found(field, line)
        columns = split(line)
        if length(columns) == 2
            info[field] = columns[2]
        elseif length(columns) == 1
            info[field] = columns
        else
            error("Could not parse line. Too many elements in:\n \"$columns\"")
        end
    end
    return nothing
end

function found(field, line)
    return occursin(field, line) && !occursin("NONE", line)
end

is_header_end(line) = occursin("HEADER_END", line)

function set_date!(info)
    if haskey(info, "AMP_CAL_DATE")
        info["date"] = info["AMP_CAL_DATE"]
    elseif haskey(info, "Calibration_DATE")
        info["date"] = info["Calibration_DATE"]
    else
        error("Was expecting either header field: 'Calibration_DATE' or 'AMP_CAL_DATE'")
    end
    return nothing
end

function format_manufacturing!(info)
    make = info["MFG"]
    make = lowercase(make)
    first_letter = uppercase(make[1])
    info["MFG"] = "$(first_letter)$(make[2:end])"
    return nothing
end

function get_data!(info)
    nlines = info["num_header_lines"]
    info["data_txt"] = info["data_in"][nlines + 1:end]
    return nothing
end
function parse_data!(info)
    info["units"] = get_units(info)
    data = parse_data_to_matrix(info)
    info["data"] = data
    return nothing
end

function get_units(info)
    if parse(Int, info["DATA_FIELDS"]) == 5
        units = ["MHz" "dBV/μPa" "V/Pa" "V^2*cm^2/W" "pF"]
    elseif parse(Int, info["DATA_FIELDS"]) == 4
        units = ["MHz" "dB" "°" "pF"]
    else
        error("Expected 'DATA_FIELDS' to take the value 4 or 5. It is: $(info["DATA_FIELDS"]).")
    end
    return units
end


function parse_data_to_matrix(info)
    data_in = info["data_txt"]
    data = split.(data_in)
    nrows = length(data)
    ncolumns = length(data[1])
    data_out = zeros(nrows, ncolumns)
    for i in 1:nrows
        line = data[i]
        if !isempty(line)
            for j in 1:ncolumns
                data_out[i,j] = parse(Float64, line[j])
            end
        end
    end
    return data_out
end

function get_raw_header!(info)
    data = info["data_in"]
    header = String[]
    for line in data
        if is_header_end(line)
            break;
        else
            push!(header, line)
        end
    end
    info["HEADER"] = header
    return nothing
end

function create_device_name!(info)
    if is_preamp(info)
        device_name = create_preamp_name(info)
    else
        device_name = create_hydrophone_name(info)
    end
    device_name = replace(device_name, "-"=>"_")
    device_name = replace(device_name, "__"=>"_")
    info["device_name"] = device_name
    return nothing
end

is_preamp(info) = haskey(info, "AMP_GAIN_NOM_DB")

function create_preamp_name(info)
    make = info["MFG"]
    model = info["MODEL"]
    sn = info["SN"]
    gain = parse(Int, info["AMP_GAIN_NOM_DB"])
    return "$(make)_$(model)_$(sn)_$(gain)dB"
end

function create_hydrophone_name(info)
    make = info["MFG"]
    model = info["MODEL"]
    sn = info["SN"]
    width = info["HYD_APERTURE_NOM_UM"]
    return "$(make)_$(model)$(width)_$(sn)"
end

function write_to_file!(info)
    device_name = info["device_name"]
    date = info["date"]
    savepath = "$(device_name)__$(date).jl"
    open(savepath, "w") do io
        write_header!(io, info)
        println(io, "HydrophoneCalibrations.@calibration :$(device_name) [")
        write_calibration_data(io, info)
        println(io, "]")
    end
    return savepath
end

function write_header!(io, info)
    println(io, "\"\"\"")
    for line in info["HEADER"]
        println(io, "$line")
    end
    println(io, "\"\"\"")
    println(io, "\n\n")
    return nothing
end

function write_calibration_data(io, info)
    ncolumns = size(info["data"],2)
    nrows = size(info["data"],1)
    units = info["units"]
    for j in 1:ncolumns
        val = info["data"][1,j]
        unit = units[j]
        print(io, "    [")
        for i in 1:nrows
            val = info["data"][i,j]
            print(io, "$(val)u\"$(unit)\", ")
        end
        println(io, "],")
    end
    return nothing
end
